//compile with gcc -Wl,--stack,8388608 -o udp.exe udp_endp.c

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/time.h>

//ports for the left channel microcontorller
#define REMOTE_PORT		12344	// port on Arduino + WiShield
#define LOCAL_PORT		12345	// port on the PC

//ports for the right channel microcontorller
#define REMOTE_PORT2	12343	// port on Arduino + WiShield
#define LOCAL_PORT2		12346	// port on the PC


volatile sig_atomic_t keep_going = 1;

//interupt for the timer
void catchInterupt(int theint){
	keep_going=0;
}

//the size in bytes of a packet
#define numSamples 325

//nubmer of songs per playlist
#define numSongs 9

int main()
{
	//have user specify the folder to look for songs in
	int playlist;
	printf("Input Desired Playlist\n[1] Classic Rock\n[2] Other\n");
	scanf("%d", &playlist);
	
	//user specifies the first song to play
	int song;
	printf("Input First Song Selection (1 through %d)\n",numSongs);
	scanf("%d", &song);
	
	//main infinite while loop
	while(1) {
		//arrays to hold each left and right packet
		unsigned char maybeL[numSamples];
		unsigned char maybeR[numSamples];
		char songTitle[35];
		sprintf(songTitle,"convertedStereo\\playlist%d\\song%d.wav",playlist,song);
		printf("Playing %s\n",songTitle);
		
		//file pointer for the WAV file
		FILE *ifp = fopen(songTitle, "r");
		
		//make sure file isn't empty
		if (ifp == NULL) {
			fprintf(stderr, "Can't open input file!\n");
			exit(1);
		}
		
		//find file size note 0L is a long 0
		fseek(ifp,0L,SEEK_END);
		int fileSize = ftell(ifp);
		
		printf("file size = %d bytes\n",fileSize);
				
		//skip the header
		fseek(ifp,44L,SEEK_SET);
				
		//declare a bunch of things for the sockets and packet sending functions
		char buf[99];
		char buf2[99];
		int sockfd, numbytes;
		int sockfd2, numbytes2;
		struct sockaddr_in remote, local;
		struct sockaddr_in remote2, local2;
		size_t addr_len;
		size_t addr_len2;
		
		memset(&remote, 0, sizeof(remote));
		memset(&remote2, 0, sizeof(remote2));

		// setup IP address of left WiShield
		remote.sin_family = AF_INET;
		remote.sin_port = htons(REMOTE_PORT);
		inet_pton(AF_INET, "192.168.1.130", &remote.sin_addr);

		// setup IP address of right WiShield
		remote2.sin_family = AF_INET;
		remote2.sin_port = htons(REMOTE_PORT2);
		inet_pton(AF_INET, "192.168.1.131", &remote2.sin_addr);

		// setup socket and connect for left channel
		if ((sockfd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
			perror("socket");
			exit(0);
		}
		
		// setup socket and connect for right channel
		if ((sockfd2 = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
			perror("socket");
			exit(0);
		}
		
		// bind to a specific port on the PC for left channel 
		local.sin_family = AF_INET;
		local.sin_port = htons(LOCAL_PORT);
		local.sin_addr.s_addr = INADDR_ANY;

		// bind to a specific port on the PC for right channel
		local2.sin_family = AF_INET;
		local2.sin_port = htons(LOCAL_PORT2);
		local2.sin_addr.s_addr = INADDR_ANY;
		
		//bind for left channel
		if ( bind(sockfd, (struct sockaddr *)&local, sizeof(local)) == -1) {
			perror("bind");
			exit(0);
		}
		
		//bind for right channel
		if ( bind(sockfd2, (struct sockaddr *)&local2, sizeof(local2)) == -1) {
			perror("bind");
			exit(0);
		}

		int lostCount=0;
		
		//this function loops over the entire file
		int indexNum;
		for(indexNum=0;indexNum<(fileSize/(2*numSamples));indexNum++){
		
			//setup the timer for 15ms interval to make sure we dont'
			//wait for an ack for too long
			keep_going=1;
			struct timeval timerValue={0,15000};
			struct timeval timerInterval={0,15000};
			struct itimerval timer={timerInterval,timerValue};
			setitimer(ITIMER_REAL, &timer, 0);
			signal(SIGALRM, (sighandler_t) catchInterupt);

			//last 3 or so samples of the song, make silent
			if(indexNum >= ((fileSize/(2*numSamples))-3)) {
				int i;
				for(i=0;i<numSamples;i++){
					maybeL[i]=0;
					maybeR[i]=0;
				}

			}
			//load each packet array up using the fgetc function
			else {
				int i;
				for(i=0;i<numSamples;i++){
					maybeL[i]=fgetc(ifp);
					maybeR[i]=fgetc(ifp);
				}
			}

			// send stuff to left WiShield
			if ((numbytes = sendto(sockfd, maybeL, numSamples, 0, (struct sockaddr *)&remote, sizeof(remote))) == -1) {
				//perror("sendto");
				//exit(1);
			}
			
			// send stuff to right WiShield
			if ((numbytes = sendto(sockfd2, maybeR, numSamples, 0, (struct sockaddr *)&remote2, sizeof(remote2))) == -1) {
				//perror("sendto");
				//exit(1);
			}		
			addr_len = sizeof(remote);
			
			//print some stuff
			printf("Packet # = %d \tPlaylist = %d\tSong= %d\n",indexNum,playlist,song);
			
			//wait for ack from left wisheild
			if ((numbytes = recvfrom(sockfd, buf, 99 , 0, (struct sockaddr *)&remote, &addr_len)) == -1) {
				//perror("recvfrom");
				//exit(1);
			}
				
		}

		//close connection
		close(sockfd);
		close(sockfd2);
		
		//close file
		fclose(ifp);
		song++;
		if(song==numSongs+1) song=1;
	}
	
	return EXIT_SUCCESS;
}
