#include <WiShield.h>
#include "TimerOne.h"

#define WIRELESS_MODE_INFRA	1
#define WIRELESS_MODE_ADHOC	2

// Wireless configuration parameters ----------------------------------------
unsigned char local_ip[] = {192,168,1,131};	// IP address of WiShield
unsigned char gateway_ip[] = {192,168,1,1};	// router or gateway IP address
unsigned char subnet_mask[] = {255,255,255,0};	// subnet mask for the local network
const prog_char ssid[] PROGMEM = {"AudioOverWifi"};		// max 32 bytes

unsigned char security_type = 0;	// 0 - open; 1 - WEP; 2 - WPA; 3 - WPA2

// WPA/WPA2 passphrase
const prog_char security_passphrase[] PROGMEM = {"Sufficiently Witty"};	// max 64 characters


//90 6E BB 2D 34 8F 00000000000000
// WEP 128-bit keys
// sample HEX keys
prog_uchar wep_keys[] PROGMEM = {0x90, 0x6E, 0xEB, 0x2D, 0x34, 0x8F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// Key 0
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	0x00,	// Key 1
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	0x00,	// Key 2
				0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	0x00	// Key 3
				};

// setup the wireless mode
// infrastructure - connect to AP
// adhoc - connect to another WiFi device
unsigned char wireless_mode = WIRELESS_MODE_INFRA;

unsigned char ssid_len;
unsigned char security_passphrase_len;
//---------------------------------------------------------------------------
volatile int globalX=3;

//size of a packet
#define BUFFERSIZE 325 //CHANGE THIS IN UDPAPP.c TOO

//define the circular buffer arrays
volatile char globalBuf1[BUFFERSIZE];
volatile char globalBuf2[BUFFERSIZE];
volatile char globalBuf3[BUFFERSIZE];

//used in the ISR
volatile int index= 0;

//ISR function that changes the PWM duty cycle each sample
void sample() {
  index++;
  //if we reach the end of an array, move to the next one
  if (index==BUFFERSIZE){ 
      index= 0;
      if(globalX==1) globalX=2; 
      else if (globalX==2) globalX=3;
      else if (globalX==3) globalX=1;
  }

  //update the PWM depending on which array we're using for reading
  if (globalX==1) OCR2B = globalBuf2[index];
  if (globalX==2) OCR2B = globalBuf3[index];
  if (globalX==3) OCR2B = globalBuf1[index];
}

//set up some stuff
void setup()
{
  pinMode(3,OUTPUT); //timer2 PWM
  pinMode(4,OUTPUT); //test LED
  pinMode(7,OUTPUT); //New Green WiFi LED
  pinMode(6,OUTPUT); //test2 LED
  pinMode(5,OUTPUT); //test2 LED
  
  //green WiFi LED off
  digitalWrite(7,LOW);
  
  //setup the WiSheild and TCP/IP stack
  WiFi.init();

  //setup the timer 2 PWM
  TCCR2A = _BV(COM2A1) | _BV(COM2B1) | _BV(WGM21) | _BV(WGM20);  // configure timer2 PWM
  TCCR2B = _BV(CS20); // set the prescaler (not sure how much, full speed?)
  OCR2A = 0; // pin 11 PWM is off
  OCR2B = 50; // turn on pin 3 PWM
  
  // set up timer1 to keep the sampling rate
  //31us for 32.258kHz
  //45us for 22.222kHz
  Timer1.initialize(45); // 125 us => 8 kHz sampling rate
  Timer1.attachInterrupt(sample);
  
  sei();
  
  //green WiFi LED on
  digitalWrite(7,HIGH);
}


void loop()
{
	//run the main WiSheild function
	//calls the TCP/IP stack and such
	WiFi.run();
}
